c   USGS Computer Program "IGRFGRID"; version 1.000
c   Technical Contact:  Ron Sweeney
c
c   Program IGRFGRID creates an output grid in latitude-longitude
c     coordinates of International Geomagnetic Reference Field
c     (IGRF) total magnetic field values, representing the earth's
c     main magnetic field, for a user-specified area.  The user
c     has a choice of models:  IGRF1965, IGRF1975, or the Definitive
c     Geomagnetic Reference Field (DGRF) package of DGRF models,
c     starting with epoch 1945, ending with epoch 1980, and coupled
c     with the IGRF 1985 model of the latest spherical harmonic
c     coefficients and secular change terms for field prediction.
c     Input elevation may be either constant barometric, or supplied
c     as a grid of barometric elevations covering an area identical
c     to the output IGRF grid.  Grid locations must be in degrees or
c     seconds of latitude & longitude (negative west longitude).
c     The output file is in USGS standard grid format.  The original
c     IGRF subroutine concept was created for the Branch of Geophysics,
c     U. S. Geological Survey, for our airborn formats by Gerald I.
c     Evenden.  Computational techniques are based on "Computation of
c     the Main Geomagnetic Field from Spherical Harmonic Expansions"
c     by J. C. Cain, et. al., NASA Data Users Note, NSSDC 68-11, May,
c     1968.
c
c   The program IGRFGRID is designed for interactive use.
c
c   Questions asked of the user:
c
c     Output file name    :  Name of output grid of IGRF magnetic
c                                 field values.
c     Title               :  56 characters of descriptive information
c                                 stored in the output grid.
c     Date of computation :  Date, entered as year & day of year, for
c                                 computation of the main field.
c     Model desired       :  Select IGRF1965,
c                                   IGRF1975, or
c                                   DIGRF, which includes ALL the DIGRF
c                                          models from 1945 - 1980, AND
c                                          including IGRF 1985.
c     Input elevation grid?  Yes -- IGRFGRID will prompt the user for a
c                                   grid file name and use the contained
c                                   elevations in the field calculations.
c                                   The output IGRF grid will have dimen-
c                                   sions identical to the elevation grid.
c                                   IGRFGRID asks if the elevations are in
c                                   feet or meters, and if the locations
c                                   are in degrees or seconds of latitude
c                                   & longitude.
c                            No  -- Constant barometric elevation is assumed
c                                   for the entire grid, and the user is
c                                   prompted for a value.  The user is also
c                                   asked to identify the dimensions of the
c                                   output IGRF grid, and the grid spacing,
c                                   in either degrees or seconds of latitude
c                                   & longitude.
c
c                       Ronald E. Sweeney
c                       Branch of Geophysics
c                       U. S. Geological Survey
c                       MS 964; Box 25046
c                       Denver, CO 80225
c
c
c
      dimension data(2000),alts(2000)
      character*56 title,etitle
      character*56 iofil,elfile
      character*8 pgm,epgm
      character*7 resp
      character*5 model
      character*4 elunx,eluny,unitss,unitsd,units
      character*1 r,w
      logical*1 elgrid,feet,deg
      data unitss/'sec '/,unitsd/'deg '/
      data r/'r'/,w/'w'/
      pgm='igrfgrid'
      rad=1./57.29578
      sec2deg=1./3600.
      ft2met=.3048
      zero=0.0
c
      nz=1
c
c   Write the ID-Stamp to default output device.
c
      write(6,17)
   17 format(/,' USGS Computer Program "IGRFGRID"; version 1.000',/,
     1       ' Technical Contact:  Ron Sweeney',/)
c
c   Request user input.
c
      write(6,1)
    1 format('$ Enter output filename:  ')
      read(5,2) iofil
    2 format(a)
      write(6,3)
    3 format('  Enter 56 character title:  ')
      read(5,2) title
      write(6,9)
    9 format('$ Enter date for computation of total field ',
     1'(yyyyddd):  ')
      read(5,*) idate
   13 write(6,10)
   10 format('$ Enter ''1965'', ''1975'', or ''digrf'' for igrf model',
     1' desired:  ')
      read(5,2) model
      if(model.ne.'1965'.and.model.ne.'1975'.and.model.ne.'digrf'
     1.and.model.ne.'DIGRF') go to 13
   21 write(6,22)
   22 format('$ Are you providing an input elevation grid ',
     1       '(''yes'' or ''no'')?  ')
      read(5,2) resp
      if(resp.ne.'yes'.and.resp.ne.'no') go to 21
      elgrid=.false.
      if(resp.eq.'yes') elgrid=.true.
c
      if(elgrid) then
c
c   Here, read parameters concerning the input elevation grid.
c
      write(6,5)
    5 format('$ Enter input barometric elevation grid filename:  ')
      read(5,2) elfile
c
c   Open the input elevation grid, & read the header record.
c
      open(unit=11,file=elfile,status='old',form='unformatted',
     1             readonly)
      read(11,end=200) etitle,epgm,ncol,nrow,nz,xo,dx,yo,dy
    6 write(6,7)
    7 format('$ Are the elevations in ''feet'' or ''meters''?  ')
      read(5,2) resp
      if(resp.ne.'feet'.and.resp.ne.'meters') go to 6
      feet=.false.
      if(resp.eq.'feet') feet=.true.
    8 write(6,12)
   12 format('$ Are the elevation locations in ''degrees'' or ',
     1'''seconds''?  ')
      read(5,2) resp
      if(resp.ne.'degrees'.and.resp.ne.'seconds') go to 8
c
      if(resp.eq.'degrees') then
         deg=.true.
         units=unitsd
      else
         deg=.false.
         units=unitss
      end if
c
      write(6,11) title,units,units,model,idate,ncol,nrow,nz,xo,dx,yo,
     1            dy
   11 format(/,2x,'title = :',a56,
     1/,2x,'unitsx = :',a4,', unitsy = :',a4,
     2/,2x,'model = ',a5,', epoch = ',i7,
     3/,2x,'ncol,nrow,nz,xo,dx,yo,dy = ',3i4,4f10.1,/)
c
c   Open the output IGRF grid file, & output the header record.
c
      open(unit=10,file=iofil,status='new',form='unformatted')
      write(10) title,pgm,ncol,nrow,nz,xo,dx,yo,dy
c
      else
c
c   Here, read parameters concerning the constant barometric elevation.
c
   26 write(6,27)
   27 format('$ Do you wish to enter geographic area in ''degrees'' ',
     1       'or ''seconds''?  ')
      read(5,2) resp
      if(resp.ne.'degrees'.and.resp.ne.'seconds') go to 26
c
      if(resp.eq.'degrees') then
         deg=.true.
         units=unitsd
      else
         deg=.false.
         units=unitss
      end if
c
      write(6,23)
   23 format('  Enter min, max lat & min, max (neg. west) lon:  ')
      read(5,*) flat1,flat2,flon1,flon2
      write(6,24)
   24 format('$ Enter grid spacing:  ')
      read(5,*) dxy
      dx=dxy
      dy=dxy
      write(6,25)
   25 format('$ Enter elevation in meters:  ')
      read(5,*) altm
      xo=flon1
      yo=flat1
      ncol=(flon2-flon1)/dxy+1.
      nrow=(flat2-flat1)/dxy+1.
c
      write(6,16) title,units,units,flat1,flat2,flon1,flon2,model,
     1idate,altm,ncol,nrow,nz,xo,dx,yo,dy
   16 format(/,2x,'title = :',a56,
     1/,2x,'unitsx = :',a4,', unitsy = :',a4,
     2/,2x,'min, max lat = (',f8.3,',',f8.3,');  min, max lon = (',
     3f10.2,',',f10.2,').',/,2x,'model = ',a5,', epoch = ',i7,
     4', alt(meters) = ',f7.1,/,2x,'ncol,nrow,nz,xo,dx,yo,dy = ',3i4,
     54f10.1,/)
c
c   Open the output IGRF grid file, & output the header record.
c
      open(10,file=iofil,status='new',form='unformatted')
      write(10) title,pgm,ncol,nrow,nz,xo,dx,yo,dy
c
      end if
c
      if(deg) go to 14
      xo=xo*sec2deg
      dx=dx*sec2deg
      yo=yo*sec2deg
      dy=dy*sec2deg
   14 if(model.ne.'1965') go to 100
c
c   igrf 1965 here.
c
      flat=yo-dy
      do 20 lt=1,nrow
      flat=flat+dy
      flatr=flat*rad
      flon=xo-dx
      if(elgrid) call rdbin(11,dum,1,alts,ncol,*200)
      do 19 ln=1,ncol
      flon=flon+dx
      flonr=flon*rad
c
      if(elgrid) then
         altm=alts(ln)
         if(feet) altm=altm*ft2met
      end if
c
      call igrf1965(flatr,flonr,altm,idate,b,1)
      data(ln)=b
   19 continue
c     write(6,15) flat,flon,altm,b
c  15 format(2x,4f10.2)
      call iorow(10,zero,1,data,ncol)
   20 continue
      go to 150
  100 if(model.ne.'1975') go to 125
c
c   igrf 1975 here.
c
      flat=yo-dy
      do 120 lt=1,nrow
      flat=flat+dy
      flatr=flat*rad
      flon=xo-dx
      if(elgrid) call rdbin(11,dum,1,alts,ncol,*200)
      do 119 ln=1,ncol
      flon=flon+dx
      flonr=flon*rad
c
      if(elgrid) then
         altm=alts(ln)
         if(feet) altm=altm*ft2met
      end if
c
      call igrf1975(flatr,flonr,altm,idate,b,1)
      data(ln)=b
  119 continue
c     write(6,15) flat,flon,altm,b
      call iorow(10,zero,1,data,ncol)
  120 continue
      go to 150
c
c   digrf here.
c
  125 flat=yo-dy
      do 130 lt=1,nrow
      flat=flat+dy
      flatr=flat*rad
      flon=xo-dx
      if(elgrid) call rdbin(11,dum,1,alts,ncol,*200)
      do 129 ln=1,ncol
      flon=flon+dx
      flonr=flon*rad
c
      if(elgrid) then
         altm=alts(ln)
         if(feet) altm=altm*ft2met
      end if
c
      call digrf(flatr,flonr,altm,idate,b,1)
      data(ln)=b
  129 continue
c     write(6,15) flat,flon,altm,b
      call iorow(10,zero,1,data,ncol)
  130 continue
  150 write(6,151)
  151 format(2x,'End of job.')
      close(10)
      if(elgrid) close(11)
      stop
c
  200 write(6,201)
  201 format(2x,'Error--unexpected E-O-F in elevation file....')
      close(10)
      close(11)
      stop
c
      end
